package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.GetAllForRelationDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.entity.ConceptInfo;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.service.impl.ConceptInfoServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.HasStaticKnowledgeVO;
import com.diagbot.vo.SearchConceptVO;
import com.diagbot.vo.StaticKnowledgeHISVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/19 14:17
 */
@Component
public class ConceptInfoFacade extends ConceptInfoServiceImpl {
    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private ConceptDetailFacade conceptDetailFacade;
    @Autowired
    private MappingConfigFacade mappingConfigFacade;

    /**
     * @param staticKnowledgeHISVO
     * @return
     */
    public List<String> getUniqueNames(StaticKnowledgeHISVO staticKnowledgeHISVO) {
        Long hospitalId = staticKnowledgeHISVO.getHospitalId();
        List<String> nameList = null;
        switch (staticKnowledgeHISVO.getType()) {
            case 1:
                Map<String, Map<String, Map<String, List<String>>>> disConfigMap
                        = mappingConfigFacade.groupByHisNameWithName(ListUtil.arrayToList(new String[] { staticKnowledgeHISVO.getHisName() }),
                        ConceptTypeEnum.Disease.getKey(), hospitalId, StatusEnum.Enable.getKey());
                if (disConfigMap != null
                        && disConfigMap.get(staticKnowledgeHISVO.getHisName()) != null) {
                    nameList = disConfigMap.get(staticKnowledgeHISVO.getHisName())
                            .get("")
                            .keySet()
                            .stream().collect(Collectors.toList());
                }
                break;
            case 2:
                Map<String, Map<String, Map<String, List<String>>>> drugConfigMap
                        = mappingConfigFacade.groupByHisNameWithName(ListUtil.arrayToList(new String[] { staticKnowledgeHISVO.getHisName() }),
                        ConceptTypeEnum.Drug.getKey(), hospitalId, StatusEnum.Enable.getKey());
                if (drugConfigMap != null
                        && drugConfigMap.get(staticKnowledgeHISVO.getHisName()) != null) {
                    nameList = drugConfigMap.get(staticKnowledgeHISVO.getHisName())
                            .get("")
                            .keySet()
                            .stream().collect(Collectors.toList());
                }
                break;
            case 3:
            case 4:
                Map<String, Map<String, Map<String, List<String>>>> lisConfigMap
                        = mappingConfigFacade.groupByHisNameWithName(ListUtil.arrayToList(new String[] { staticKnowledgeHISVO.getHisName() }),
                        ConceptTypeEnum.LisPack.getKey(), hospitalId, StatusEnum.Enable.getKey());
                if (lisConfigMap != null) {
                    if (StringUtil.isBlank(staticKnowledgeHISVO.getHisDetailName())) {
                        staticKnowledgeHISVO.setHisDetailName("");
                    }
                    if (lisConfigMap.containsKey(staticKnowledgeHISVO.getHisName())) {
                        if (lisConfigMap.get(staticKnowledgeHISVO.getHisName()).containsKey(staticKnowledgeHISVO.getHisDetailName())) {
                            nameList = lisConfigMap.get(staticKnowledgeHISVO.getHisName())
                                    .get(staticKnowledgeHISVO.getHisDetailName())
                                    .keySet()
                                    .stream().collect(Collectors.toList());
                        }
                    }
                }
                break;
            case 5:
            case 6:
                Map<String, Map<String, Map<String, List<String>>>> pacsConfigMap
                        = mappingConfigFacade.groupByHisNameWithName(ListUtil.arrayToList(new String[] { staticKnowledgeHISVO.getHisName() }),
                        ConceptTypeEnum.Pacs.getKey(), hospitalId, StatusEnum.Enable.getKey());
                if (pacsConfigMap != null
                        && pacsConfigMap.get(staticKnowledgeHISVO.getHisName()) != null) {
                    nameList = pacsConfigMap.get(staticKnowledgeHISVO.getHisName())
                            .get("")
                            .keySet()
                            .stream().collect(Collectors.toList());
                }
                break;
            case 7:
                Map<String, Map<String, Map<String, List<String>>>> operationConfigMap
                        = mappingConfigFacade.groupByHisNameWithName(ListUtil.arrayToList(new String[] { staticKnowledgeHISVO.getHisName() }),
                        ConceptTypeEnum.Operation.getKey(), hospitalId, StatusEnum.Enable.getKey());
                if (operationConfigMap != null &&
                        operationConfigMap.get(staticKnowledgeHISVO.getHisName()) != null) {
                    nameList = operationConfigMap.get(staticKnowledgeHISVO.getHisName())
                            .get("")
                            .keySet()
                            .stream().collect(Collectors.toList());
                }
                break;
            case 8:
                Map<String, Map<String, Map<String, List<String>>>> scaleConfigMap
                        = mappingConfigFacade.groupByHisNameWithName(ListUtil.arrayToList(new String[] { staticKnowledgeHISVO.getHisName() }),
                        ConceptTypeEnum.Scale.getKey(), hospitalId, StatusEnum.Enable.getKey());
                if (scaleConfigMap != null &&
                        scaleConfigMap.get(staticKnowledgeHISVO.getHisName()) != null) {
                    nameList = scaleConfigMap.get(staticKnowledgeHISVO.getHisName())
                            .get("")
                            .keySet()
                            .stream().collect(Collectors.toList());
                }
                break;
            case 9:
                Map<String, Map<String, Map<String, List<String>>>> nurseConfigMap
                        = mappingConfigFacade.groupByHisNameWithName(ListUtil.arrayToList(new String[] { staticKnowledgeHISVO.getHisName() }),
                        ConceptTypeEnum.Nurse.getKey(), hospitalId, StatusEnum.Enable.getKey());
                if (nurseConfigMap != null &&
                        nurseConfigMap.get(staticKnowledgeHISVO.getHisName()) != null) {
                    nameList = nurseConfigMap.get(staticKnowledgeHISVO.getHisName())
                            .get("")
                            .keySet()
                            .stream().collect(Collectors.toList());
                }
                break;
            default:
                break;

        }
        if (ListUtil.isNotEmpty(nameList)) {
            nameList = nameList
                    .stream()
                    .distinct()
                    .collect(Collectors.toList());
        }
        return nameList;
    }

    /**
     * 批量更新静态知识标志
     *
     * @return
     */
    public Boolean updateHasInfoStatusBatch() {
        QueryWrapper<ConceptInfo> conceptInfoQueryWrapper = new QueryWrapper<ConceptInfo>();
        conceptInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
        List<ConceptInfo> conceptInfoList = this.list(conceptInfoQueryWrapper);
        if (ListUtil.isEmpty(conceptInfoList)) {
            return true;
        }

        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<ConceptDetail>();
        conceptDetailQueryWrapper.select("distinct concept_id");
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey());
        List<ConceptDetail> conceptDetailList = conceptDetailFacade.list(conceptDetailQueryWrapper);
        Map<Long, List<ConceptDetail>> detailMap =
                EntityUtil.makeEntityListMap(conceptDetailList, "conceptId");

        List<ConceptInfo> modifyList = Lists.newLinkedList();
        conceptInfoList.forEach(item -> {
            if (detailMap.containsKey(item.getId())) {
                modifyList.add(item);
            }
        });

        for (ConceptInfo conceptInfo : modifyList) {
            //TODO 更新图谱是否有静态信息
            HasStaticKnowledgeVO hasStaticKnowledgeVO = new HasStaticKnowledgeVO();
            hasStaticKnowledgeVO.setName(conceptInfo.getName());
            hasStaticKnowledgeVO.setType(conceptInfo.getType());
            hasStaticKnowledgeVO.setHasInfo(1);
            RespDTO<Boolean> respDTO = cdssCoreClient.updateHasInfoStatus(hasStaticKnowledgeVO);
        }
        return true;
    }

    public List<GetAllForRelationDTO> searchConceptByNameAndLibType(SearchConceptVO searchConceptVO) {
        RespDTO<List<GetAllForRelationDTO>> relationDTORespDTO = cdssCoreClient.searchConcept(searchConceptVO);
        RespDTOUtil.respNGDeal(relationDTORespDTO, "查询诊断依据相关的类型术语失败");
        return relationDTORespDTO.data;
    }

    public List<GetAllForRelationDTO> addConceptClass(SearchConceptVO searchConceptVO) {
        RespDTO<List<GetAllForRelationDTO>> relationDTORespDTO = cdssCoreClient.addConceptClass(searchConceptVO);
        RespDTOUtil.respNGDeal(relationDTORespDTO, "术语集合新增失败");
        return relationDTORespDTO.data;
    }

    public List<GetAllForRelationDTO> searchConceptRuleClass(SearchConceptVO searchConceptVO) {
        RespDTO<List<GetAllForRelationDTO>> relationDTORespDTO = cdssCoreClient.searchConceptRuleClass(searchConceptVO);
        RespDTOUtil.respNGDeal(relationDTORespDTO, "查询规则维护术语查询术语失败");
        return relationDTORespDTO.data;
    }
}
